@@ -1,7 +1,7 @@
-#' Compile a LaTeX document to PDF
+#' Compile a LaTeX document
 #'
 #' The function \code{latexmk()} emulates the system command \command{latexmk}
-#' (\url{https://ctan.org/pkg/latexmk}) to compile a LaTeX document to PDF. The
+#' (\url{https://ctan.org/pkg/latexmk}) to compile a LaTeX document. The
 #' functions \code{pdflatex()}, \code{xelatex()}, and \code{lualatex()} are
 #' wrappers of \code{latexmk(engine =, emulation = TRUE)}.
 #'
@@ -55,15 +55,16 @@
 #'   packages found by \code{\link{parse_packages}()} from the LaTeX log. This
 #'   argument is only for the emulation mode and TeX Live.
 #' @param pdf_file Path to the PDF output file. By default, it is under the same
-#'   directory as the input \code{file} and also has the same base name.
+#'   directory as the input \code{file} and also has the same base name. When
+#'   \code{engine == 'latex'}, this will be a DVI file.
 #' @param clean Whether to clean up auxiliary files after compilation (can be
 #'   set in the global option \code{tinytex.clean}, which defaults to
 #'   \code{TRUE}).
 #' @export
-#' @return A character string of the path of the PDF output file (i.e., the
-#'   value of the \code{pdf_file} argument).
+#' @return A character string of the path of the output file (i.e., the value of
+#'   the \code{pdf_file} argument).
 latexmk = function(
-  file, engine = c('pdflatex', 'xelatex', 'lualatex'),
+  file, engine = c('pdflatex', 'xelatex', 'lualatex', 'latex'),
   bib_engine = c('bibtex', 'biber'), engine_args = NULL, emulation = TRUE,
   max_times = 10, install_packages = emulation && tlmgr_available(),
   pdf_file = gsub('tex$', 'pdf', file), clean = TRUE
@@ -74,6 +75,7 @@ latexmk = function(
   if (missing(engine)) engine = getOption('tinytex.engine', engine)
   engine = gsub('^(pdf|xe|lua)(tex)$', '\\1la\\2', engine)  # normalize *tex to *latex
   engine = match.arg(engine)
+  is_latex = engine == 'latex'
   tweak_path()
   if (missing(emulation)) emulation = getOption('tinytex.latexmk.emulation', emulation)
   if (!emulation) {
@@ -89,9 +91,8 @@ latexmk = function(
   if (missing(bib_engine)) bib_engine = getOption('tinytex.bib_engine', bib_engine)
   if (missing(engine_args)) engine_args = getOption('tinytex.engine_args', engine_args)
   if (missing(clean)) clean = getOption('tinytex.clean', TRUE)
-  pdf = gsub('[.]tex$', '.pdf', basename(file))
-  output_dir = getOption('tinytex.output_dir')
-  if (!is.null(output_dir)) {
+  pdf = gsub('tex$', if (is_latex) 'dvi' else 'pdf', basename(file))
+  if (!is.null(output_dir <- getOption('tinytex.output_dir'))) {
     output_dir_arg = shQuote(paste0(if (emulation) '-', '-output-directory=', output_dir))
     if (length(grep(output_dir_arg, engine_args, fixed = TRUE)) == 0) stop(
       "When you set the global option 'tinytex.output_dir', the argument 'engine_args' ",
@@ -100,8 +101,9 @@ latexmk = function(
     pdf = file.path(output_dir, pdf)
     if (missing(pdf_file)) pdf_file = file.path(output_dir, basename(pdf_file))
   }
+  if (is_latex) pdf_file = with_ext(pdf_file, 'dvi')
   check_pdf = function() {
-    if (!file.exists(pdf)) show_latex_error(file, sub('.pdf$', '.log', pdf))
+    if (!file.exists(pdf)) show_latex_error(file, with_ext(pdf, 'log'), TRUE)
     file_rename(pdf, pdf_file)
     pdf_file
   }
@@ -110,8 +112,9 @@ latexmk = function(
     return(check_pdf())
   }
   system2_quiet('latexmk', c(
-    '-pdf -latexoption=-halt-on-error -interaction=batchmode',
-    paste0('-pdflatex=', engine), engine_args, shQuote(file)
+    '-latexoption=-halt-on-error -interaction=batchmode',
+    if (is_latex) '-latex=latex' else paste0('-pdf -pdflatex=', engine),
+    engine_args, shQuote(file)
   ), error = {
     if (install_packages) warning(
       'latexmk(install_packages = TRUE) does not work when emulation = FALSE'
@@ -149,6 +152,7 @@ latexmk_emu = function(
   )
   base = gsub('[.]tex$', '', basename(file))
   aux_files = paste(base, aux, sep = '.')
+  aux_files = c(aux_files, 'preview.aux')  # generated by the preview package
   if (!is.null(output_dir <- getOption('tinytex.output_dir')))
     aux_files = file.path(output_dir, aux_files)
   logfile = aux_files[1]; unlink(logfile)  # clean up the log before compilation
@@ -159,12 +163,12 @@ latexmk_emu = function(
   on.exit({
     files2 = exist_files(aux_files)
     files3 = setdiff(files2, files1)
-    if (keep_log || latex_warning(logfile)) files3 = setdiff(files3, logfile)
+    if (keep_log || length(latex_warning(logfile, TRUE))) files3 = setdiff(files3, logfile)
     if (clean) unlink(files3)
   }, add = TRUE)
 
   pkgs_last = character()
-  filep = sub('.log$', '.pdf', logfile)
+  filep = sub('.log$', if (engine == 'latex') '.dvi' else '.pdf', logfile)
   run_engine = function() {
     on_error  = function() {
       if (install_packages && file.exists(logfile)) {
@@ -189,6 +193,7 @@ latexmk_emu = function(
     invisible(res)
   }
   run_engine()
+  if (install_packages) check_babel(logfile)
   # generate index
   if (file.exists(idx <- aux_files[2])) {
     idx_engine = getOption('tinytex.makeindex', 'makeindex')
@@ -283,11 +288,14 @@ use_file_stdout = function() {
 }
 
 # parse the LaTeX log and show error messages
-show_latex_error = function(file, logfile = gsub('[.]tex$', '.log', basename(file))) {
-  e = c('Failed to compile ', file, '.')
+show_latex_error = function(
+  file, logfile = gsub('[.]tex$', '.log', basename(file)), force = FALSE
+) {
+  e = c('Failed to compile ', file, '. See https://yihui.name/tinytex/r/#debugging for debugging tips.')
   if (!file.exists(logfile)) stop(e, call. = FALSE)
   x = readLines(logfile, warn = FALSE)
   b = grep('^\\s*$', x)  # blank lines
+  b = c(b, which(x == "Here is how much of TeX's memory you used:"))
   m = NULL
   for (i in grep('^! ', x)) {
     # ignore the last error message about the fatal error
@@ -298,17 +306,50 @@ show_latex_error = function(file, logfile = gsub('[.]tex$', '.log', basename(fil
   }
   if (length(m)) {
     message(paste(m, collapse = '\n'))
+    check_inline_math(m, file)
     stop(e, ' See ', logfile, ' for more info.', call. = FALSE)
-  }
+  } else if (force) stop(e, call. = FALSE)
 }
 
-# whether a LaTeX log file contains warnings
-latex_warning = function(file) {
-  if (!file.exists(file)) return(FALSE)
+check_inline_math = function(x, f) {
+  r = 'l[.][0-9]+\\s*|\\s*[0-9.]+\\\\times.*'
+  if (!any('! Missing $ inserted.' == x) || !length(i <- grep(r, x))) return()
+  m = gsub(r, '', x[i]); m = m[m != '']
+  s = xfun::with_ext(f, 'Rmd')
+  if (file.exists(s)) message(
+    if (length(m)) c('Try to find the following text in ', s, ':\n', paste(' ', m, '\n'), '\n'),
+    'You may need to add $ $ around a certain inline R expression `r ` in ', s,
+    if (length(m)) ' (see the above hint)',
+    '. See https://github.com/rstudio/rmarkdown/issues/385 for more info.'
+  )
+}
+
+# whether a LaTeX log file contains LaTeX or package (e.g. babel) warnings
+latex_warning = function(file, show = FALSE) {
+  if (!file.exists(file)) return()
   x = readLines(file, warn = FALSE)
-  if (length(i <- grep(r <- '^LaTeX Warning:', x)) == 0) return(FALSE)
-  warning(paste(c('LaTeX Warning(s):', gsub(r, ' ', x[i])), collapse = '\n'), call. = FALSE)
-  TRUE
+  if (length(i <- grep('^(LaTeX|Package [[:alnum:]]+) Warning:', x)) == 0) return()
+  # these warnings may be okay (our Pandoc LaTeX template in rmarkdown may need an update)
+  i = i[grep('^Package (fixltx2e|caption|hyperref) Warning:', x[i], invert = TRUE)]
+  if (length(i) == 0) return()
+  b = grep('^\\s*$', x)
+  i = unlist(lapply(i, function(j) {
+    n = b[b > j]
+    n = if (length(n) == 0) i else min(n) - 1L
+    j:n
+  }))
+  i = sort(unique(i))
+  if (show) warning(paste(x[i], collapse = '\n'), call. = FALSE)
+  x[i]
+}
+
+# check if any babel packages are missing
+check_babel = function(file) {
+  if (length(m <- latex_warning(file)) == 0 || length(grep('^Package babel Warning:', m)) == 0)
+    return()
+  r = "^\\(babel).* language `([[:alpha:]]+)'.*$"
+  if (length(i <- grep(r, m)) == 0) return()
+  tlmgr_install(paste0('hyphen-', tolower(gsub(r, '\\1', m[i]))))
 }
 
 # check the version of latexmk
@@ -363,7 +404,8 @@ file_rename = function(from, to) {
 parse_packages = function(
   log, text = readLines(log), files = detect_files(text), quiet = rep(FALSE, 3)
 ) {
-  pkgs = character(); quiet = rep_len(quiet, length.out = 3); x = files
+  pkgs = character(); quiet = rep_len(quiet, length.out = 3)
+  x = unique(c(files, miss_font()))
   if (length(x) == 0) {
     if (!quiet[1]) message(
       'I was unable to find any missing LaTeX packages from the error log',
@@ -431,10 +473,7 @@ detect_files = function(text) {
     v = gsub(p, '\\1', z)
     if (length(v) == 0) return(v)
     if (!(p %in% r[1:4])) return(if (p == r[5]) 'epstopdf' else v)
-    if (p == r[4]) return(paste0(v, '.sty'))
-    i = !grepl('[.]', v)
-    v[i] = paste0(v[i], '[.](tfm|afm|mf|otf)')
-    v
+    if (p == r[4]) paste0(v, '.sty') else font_ext(v)
   })))
 }
 
@@ -443,6 +482,21 @@ parse_install = function(...) {
   tlmgr_install(parse_packages(...))
 }
 
+# check missfont.log and detect the missing font packages
+miss_font = function() {
+  if (!file.exists(f <- 'missfont.log')) return()
+  on.exit(unlink(f), add = TRUE)
+  x = gsub('\\s*$', '', readLines(f))
+  x = unique(gsub('.+\\s+', '', x))
+  if (length(x)) font_ext(x)
+}
+
+font_ext = function(x) {
+  i = !grepl('[.]', x)
+  x[i] = paste0(x[i], '[.](tfm|afm|mf|otf)')
+  x
+}
+
 # it should be rare that we need to manually run texhash
 texhash = function() {
   tweak_path()
